selenium_wrapper_web_driver_not_found_error = "Web Driver not found"

selenium_wrapper_set_options_error = "only accept dict type"
selenium_wrapper_set_argument_error = "only accept str type"

executor_data_error = "executor receive wrong data"
executor_list_error = "executor receive wrong data list is none or wrong type"
# json
cant_find_json_error = "can't find json"
cant_save_json_error = "can't save json"
cant_reformat_json_error = "can't reformat json is type right?"
cant_find_element_in_json_error = "can't find element in json"
json_type_error = "json type error"
wrong_json_data_error = "can't parser json"
cant_generate_json_report: str = "can't generate json report"

html_generate_no_data_tag = "can't make html report, data is null?"
# add command
add_command_exception_tag = "command value type should be as method or function"
# argparse
argparse_get_wrong_data = "argparse receive wrong data"
# XML
cant_read_xml_error: str = "can't read xml"
xml_type_error: str = "xml type error"
# Callback executor
get_bad_trigger_method: str = "get bad trigger method, only accept kwargs and args"
get_bad_trigger_function: str = "get bad trigger function only accept function in event_dict"
